@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package gnu.javax.swing.text.html.parser;
 
+import gnu.java.lang.CPStringBuilder;
+
 import gnu.javax.swing.text.html.parser.models.node;
 import gnu.javax.swing.text.html.parser.models.transformer;
 
@@ -153,7 +155,7 @@ public abstract class htmlValidator
    * Remove the given tag from the stack or (if found) from the list
    * of the forcibly closed tags.
    */
-  public void closeTag(TagElement tElement)
+  public boolean closeTag(TagElement tElement)
   {
     HTML.Tag tag = tElement.getHTMLTag();
     hTag x;
@@ -191,11 +193,12 @@ public abstract class htmlValidator
                   }
 
                 stack.remove(x);
-                return;
+                return true;
               }
           }
       }
     s_error("Closing unopened <" + tag + ">");
+    return false;
   }
 
   /**
@@ -233,7 +236,9 @@ public abstract class htmlValidator
                 Element fe = (Element) v;
 
                 // notify the content model that we add the proposed tag
-                getCurrentContentModel().show(fe);
+                node ccm = getCurrentContentModel();
+                if (ccm != null)
+                  ccm.show(fe);
                 openFictionalTag(fe);
 
                 Object vv = tagIsValidForContext(tElement);
@@ -301,13 +306,12 @@ public abstract class htmlValidator
   }
 
   /**
-   * Check if this tag is valid for the current context.
-   * Return Boolean.True if it is OK, Boolean.False
-   * if it is surely not OK or the Element that the
-   * content model recommends to insert making the situation
-   * ok. If Boolean.True is returned, the content model current
-   * position is moved forward. Otherwise this position remains
-   * the same.
+   * Check if this tag is valid for the current context. Return Boolean.True if
+   * it is OK, Boolean.False if it is surely not OK or the Element that the
+   * content model recommends to insert making the situation ok. If Boolean.True
+   * is returned, the content model current position is moved forward. Otherwise
+   * this position remains the same.
+   * 
    * @param tElement
    * @return
    */
@@ -321,7 +325,7 @@ public abstract class htmlValidator
 
     // Check exclusions and inclusions.
     ListIterator iter = stack.listIterator(stack.size());
-    hTag t;
+    hTag t = null;
     final int idx = tElement.getElement().index;
 
     // Check only known tags.
@@ -331,19 +335,27 @@ public abstract class htmlValidator
         while (iter.hasPrevious())
           {
             t = (hTag) iter.previous();
-            if (!t.forcibly_closed)
+            if (! t.forcibly_closed)
               {
-                if (t.element.exclusions != null &&
-                    t.element.exclusions.get(idx)
-                   )
+                if (t.element.exclusions != null
+                    && t.element.exclusions.get(idx))
                   return Boolean.FALSE;
 
                 if (t.element.inclusions != null)
                   inclusions.or(t.element.inclusions);
               }
           }
-        if (!inclusions.get(idx))
-          return Boolean.FALSE;
+        if (! inclusions.get(idx))
+          {
+            // If we need to insert something, and cannot do this, but
+            // it is allowed to insert the paragraph here, insert the
+            // paragraph.
+            Element P = dtd.getElement(HTML_401F.P);
+            if (inclusions.get(P.index))
+              return P;
+            else
+              return Boolean.FALSE;
+          }
       }
     return Boolean.TRUE;
   }
@@ -489,11 +501,11 @@ public abstract class htmlValidator
     dtdAttribute = tag.getElement().getAttribute(foundAttribute.toString());
     if (dtdAttribute == null)
       {
-        StringBuffer valid =
-          new StringBuffer("The tag <" + tag.getHTMLTag() +
-                           "> cannot contain the attribute '" + foundAttribute +
-                           "'. The valid attributes for this tag are: "
-                          );
+        CPStringBuilder valid =
+          new CPStringBuilder("The tag <" + tag.getHTMLTag() +
+			      "> cannot contain the attribute '" + foundAttribute +
+			      "'. The valid attributes for this tag are: "
+			      );
 
         AttributeList a = tag.getElement().getAttributes();
 
@@ -535,22 +547,22 @@ public abstract class htmlValidator
         !dtdAttribute.values.contains(value.toUpperCase())
        )
       {
-        StringBuffer valid;
+        CPStringBuilder valid;
         if (dtdAttribute.values.size() == 1)
           valid =
-            new StringBuffer("The attribute '" + foundAttribute +
-                             "' of the tag <" + tag.getHTMLTag() +
-                             "> cannot have the value '" + value +
-                             "'. The only valid value is "
-                            );
+            new CPStringBuilder("The attribute '" + foundAttribute +
+				"' of the tag <" + tag.getHTMLTag() +
+				"> cannot have the value '" + value +
+				"'. The only valid value is "
+				);
         else
           valid =
-            new StringBuffer("The attribute '" + foundAttribute +
-                             "' of the tag <" + tag.getHTMLTag() +
-                             "> cannot have the value '" + value + "'. The " +
-                             dtdAttribute.values.size() +
-                             " valid values are: "
-                            );
+            new CPStringBuilder("The attribute '" + foundAttribute +
+				"' of the tag <" + tag.getHTMLTag() +
+				"> cannot have the value '" + value + "'. The " +
+				dtdAttribute.values.size() +
+				" valid values are: "
+				);
 
         Enumeration vv = dtdAttribute.values.elements();
         while (vv.hasMoreElements())
