@@ -1,5 +1,5 @@
 /* GridBagLayout - Layout manager for components according to GridBagConstraints
-   Copyright (C) 2002, 2003, 2004, 2005  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005, 2006  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -46,6 +46,7 @@ import java.util.Hashtable;
 /**
  * @author Michael Koch (konqueror@gmx.de)
  * @author Jeroen Frijters (jeroen@frijters.net)
+ * @author Andrew John Hughes (gnu_andrew@member.fsf.org)
  */
 public class GridBagLayout
     implements Serializable, LayoutManager2
@@ -62,8 +63,8 @@ public class GridBagLayout
     // REMAINDER constraints.
     // Constraints kept in comptable are never modified, and constraints
     // kept in internalcomptable can be modified internally only.
-    protected Hashtable comptable;
-    private Hashtable internalcomptable;
+    protected Hashtable<Component,GridBagConstraints> comptable;
+    private Hashtable<Component,GridBagConstraints> internalcomptable;
     protected GridBagLayoutInfo layoutInfo;
     protected GridBagConstraints defaultConstraints;
 
@@ -74,8 +75,8 @@ public class GridBagLayout
 
     public GridBagLayout ()
     {
-	this.comptable = new Hashtable();
-	this.internalcomptable = new Hashtable();
+	this.comptable = new Hashtable<Component,GridBagConstraints>();
+	this.internalcomptable = new Hashtable<Component,GridBagConstraints>();
 	this.defaultConstraints= new GridBagConstraints();
     }
 
@@ -210,12 +211,12 @@ public class GridBagLayout
 
     protected GridBagConstraints lookupConstraints (Component component)
     {
-	GridBagConstraints result = (GridBagConstraints) comptable.get (component);
+	GridBagConstraints result = comptable.get (component);
 
 	if (result == null)
 	{
 	    setConstraints (component, defaultConstraints);
-	    result = (GridBagConstraints) comptable.get (component);
+	    result = comptable.get (component);
 	}
     
 	return result;
@@ -223,8 +224,7 @@ public class GridBagLayout
 
     private GridBagConstraints lookupInternalConstraints (Component component)
     {
-	GridBagConstraints result =
-            (GridBagConstraints) internalcomptable.get (component);
+	GridBagConstraints result = internalcomptable.get (component);
 
 	if (result == null)
 	{
@@ -320,12 +320,34 @@ public class GridBagLayout
     }
 
     /**
-     * Obsolete.
+     * Return a string representation of this GridBagLayout.
+     *
+     * @return a string representation
      */
-    protected void AdjustForGravity (GridBagConstraints gbc, Rectangle rect)
+    public String toString()
     {
-      // FIXME
-      throw new Error ("Not implemented");
+      return getClass().getName();
+    }
+    
+    /**
+     * Move and resize a rectangle according to a set of grid bag
+     * constraints.  The x, y, width and height fields of the
+     * rectangle argument are adjusted to the new values.
+     *
+     * @param constraints position and size constraints
+     * @param r rectangle to be moved and resized
+     */
+    protected void AdjustForGravity (GridBagConstraints constraints,
+                                     Rectangle r)
+    {
+      Insets insets = constraints.insets;
+      if (insets != null)
+	{
+	  r.x += insets.left;
+	  r.y += insets.top;
+	  r.width -= insets.left + insets.right;
+	  r.height -= insets.top + insets.bottom;
+	}
     }
 
     /**
@@ -341,109 +363,122 @@ public class GridBagLayout
       GridBagLayoutInfo info = getLayoutInfo (parent, PREFERREDSIZE);
       if (info.cols == 0 && info.rows == 0)
         return;
-      layoutInfo = info;
 
       // DEBUG
-      //dumpLayoutInfo (layoutInfo);
-    
-      for(int i = 0; i < components.length; i++)
-	{
-          Component component = components [i];
-		
-          // If component is not visible we dont have to care about it.
-          if (!component.isVisible())
-            continue;
-		
-          GridBagConstraints constraints =
-              lookupInternalConstraints(component);
+      //dumpLayoutInfo (info);
 
-          int cellx = sumIntArray(layoutInfo.colWidths, constraints.gridx);
-          int celly = sumIntArray(layoutInfo.rowHeights, constraints.gridy);
-          int cellw = sumIntArray(layoutInfo.colWidths,
-                                  constraints.gridx + constraints.gridwidth) - cellx;
-          int cellh = sumIntArray(layoutInfo.rowHeights,
-                                  constraints.gridy + constraints.gridheight) - celly;
+      // Calling setBounds on these components causes this layout to
+      // be invalidated, clearing the layout information cache,
+      // layoutInfo.  So we wait until after this for loop to set
+      // layoutInfo.
+      Component lastComp = null;
 
-          Insets insets = constraints.insets;
-          if (insets != null)
-	    {
-              cellx += insets.left;
-              celly += insets.top;
-              cellw -= insets.left + insets.right;
-              cellh -= insets.top + insets.bottom;
-	    }
+      Rectangle cell = new Rectangle();
 
-          Dimension dim = component.getPreferredSize();
+      for (int i = 0; i < components.length; i++)
+      {
+        Component component = components[i];
 
-          // Note: Documentation says that padding is added on both sides, but
-          // visual inspection shows that the Sun implementation only adds it
-          // once, so we do the same.
-          dim.width += constraints.ipadx;
-          dim.height += constraints.ipady;
+        // If component is not visible we dont have to care about it.
+        if (! component.isVisible())
+          continue;
 
-          switch(constraints.fill)
-	    {
-            case GridBagConstraints.HORIZONTAL:
-              dim.width = cellw;
-              break;
-            case GridBagConstraints.VERTICAL:
-              dim.height = cellh;
-              break;
-            case GridBagConstraints.BOTH:
-              dim.width = cellw;
-              dim.height = cellh;
-              break;
-	    }
+        Dimension dim = component.getPreferredSize();
+        GridBagConstraints constraints = lookupInternalConstraints(component);
+        
+        if (lastComp != null
+            && constraints.gridheight == GridBagConstraints.REMAINDER)
+          cell.y += cell.height;
+        else
+          cell.y = sumIntArray(info.rowHeights, constraints.gridy);
+        
+        if (lastComp != null
+            && constraints.gridwidth == GridBagConstraints.REMAINDER)
+          cell.x += cell.width;
+        else
+          cell.x = sumIntArray(info.colWidths, constraints.gridx);
 
-          int x;
-          int y;
+        cell.width = sumIntArray(info.colWidths, constraints.gridx
+                                            + constraints.gridwidth) - cell.x;
+        cell.height = sumIntArray(info.rowHeights, constraints.gridy
+                                             + constraints.gridheight) - cell.y;
+	
+        // Adjust for insets.
+ 	AdjustForGravity( constraints, cell );
 
-          switch(constraints.anchor)
-	    {
-            case GridBagConstraints.NORTH:
-              x = cellx + (cellw - dim.width) / 2;
-              y = celly;
-              break;
-            case GridBagConstraints.SOUTH:
-              x = cellx + (cellw - dim.width) / 2;
-              y = celly + cellh - dim.height;
-              break;
-            case GridBagConstraints.WEST:
-              x = cellx;
-              y = celly + (cellh - dim.height) / 2;
-              break;
-            case GridBagConstraints.EAST:
-              x = cellx + cellw - dim.width;
-              y = celly + (cellh - dim.height) / 2;
-              break;
-            case GridBagConstraints.NORTHEAST:
-              x = cellx + cellw - dim.width;
-              y = celly;
-              break;
-            case GridBagConstraints.NORTHWEST:
-              x = cellx;
-              y = celly;
-              break;
-            case GridBagConstraints.SOUTHEAST:
-              x = cellx + cellw - dim.width;
-              y = celly + cellh - dim.height;
-              break;
-            case GridBagConstraints.SOUTHWEST:
-              x = cellx;
-              y = celly + cellh - dim.height;
-              break;
-            default:
-              x = cellx + (cellw - dim.width) / 2;
-              y = celly + (cellh - dim.height) / 2;
-              break;
-	    }
+        // Note: Documentation says that padding is added on both sides, but
+        // visual inspection shows that the Sun implementation only adds it
+        // once, so we do the same.
+        dim.width += constraints.ipadx;
+        dim.height += constraints.ipady;
 
-          component.setBounds(layoutInfo.pos_x + x, layoutInfo.pos_y + y, dim.width, dim.height);
-	}
+        switch (constraints.fill)
+          {
+          case GridBagConstraints.HORIZONTAL:
+            dim.width = cell.width;
+            break;
+          case GridBagConstraints.VERTICAL:
+            dim.height = cell.height;
+            break;
+          case GridBagConstraints.BOTH:
+            dim.width = cell.width;
+            dim.height = cell.height;
+            break;
+          }
 
-      // DEBUG
-      //dumpLayoutInfo (layoutInfo);
-    }
+        int x = 0;
+        int y = 0;
+
+        switch (constraints.anchor)
+          {
+          case GridBagConstraints.NORTH:
+            x = cell.x + (cell.width - dim.width) / 2;
+            y = cell.y;
+            break;
+          case GridBagConstraints.SOUTH:
+            x = cell.x + (cell.width - dim.width) / 2;
+            y = cell.y + cell.height - dim.height;
+            break;
+          case GridBagConstraints.WEST:
+            x = cell.x;
+            y = cell.y + (cell.height - dim.height) / 2;
+            break;
+          case GridBagConstraints.EAST:
+            x = cell.x + cell.width - dim.width;
+            y = cell.y + (cell.height - dim.height) / 2;
+            break;
+          case GridBagConstraints.NORTHEAST:
+            x = cell.x + cell.width - dim.width;
+            y = cell.y;
+            break;
+          case GridBagConstraints.NORTHWEST:
+            x = cell.x;
+            y = cell.y;
+            break;
+          case GridBagConstraints.SOUTHEAST:
+            x = cell.x + cell.width - dim.width;
+            y = cell.y + cell.height - dim.height;
+            break;
+          case GridBagConstraints.SOUTHWEST:
+            x = cell.x;
+            y = cell.y + cell.height - dim.height;
+            break;
+          default:
+            x = cell.x + (cell.width - dim.width) / 2;
+            y = cell.y + (cell.height - dim.height) / 2;
+            break;
+          }
+        component.setBounds(info.pos_x + x, info.pos_y + y, dim.width,
+                            dim.height);
+        lastComp = component;
+      }
+
+    // DEBUG
+    //dumpLayoutInfo(info);
+
+    // Cache layout information.
+    layoutInfo = getLayoutInfo(parent, PREFERREDSIZE);
+  }
 
     /**
      * Obsolete.
@@ -464,26 +499,27 @@ public class GridBagLayout
 
       // Guaranteed to contain the last component added to the given row
       // or column, whose gridwidth/height is not REMAINDER.
-      HashMap lastInRow = new HashMap();
-      HashMap lastInCol = new HashMap();
+      HashMap<Integer,Component> lastInRow = new HashMap<Integer,Component>();
+      HashMap<Integer,Component> lastInCol = new HashMap<Integer,Component>();
 
       Component[] components = parent.getComponents();
 
       // Components sorted by gridwidths/heights,
       // smallest to largest, with REMAINDER and RELATIVE at the end.
       // These are useful when determining sizes and weights.
-      ArrayList sortedByWidth = new ArrayList(components.length);
-      ArrayList sortedByHeight = new ArrayList(components.length);
+      ArrayList<Component> sortedByWidth =
+	new ArrayList<Component>(components.length);
+      ArrayList<Component> sortedByHeight =
+	new ArrayList<Component>(components.length);
 
       // STEP 1: first we figure out how many rows/columns
       for (int i = 0; i < components.length; i++)
 	{
           Component component = components [i];
-		
           // If component is not visible we dont have to care about it.
           if (!component.isVisible())
             continue;
-		
+
           // When looking up the constraint for the first time, check the
           // original unmodified constraint.  After the first time, always
           // refer to the internal modified constraint.
@@ -510,7 +546,6 @@ public class GridBagLayout
           //
           //       nothing to check; just add it
 
-
           // cases 1 and 2
           if(constraints.gridx == GridBagConstraints.RELATIVE)
             {
@@ -526,7 +561,7 @@ public class GridBagLayout
                 x = 0;
               else
                 {
-                  Component lastComponent = (Component) lastInRow.get(new Integer(constraints.gridy));
+                  Component lastComponent = lastInRow.get(new Integer(constraints.gridy));
                   GridBagConstraints lastConstraints = lookupInternalConstraints(lastComponent);
                   x = lastConstraints.gridx + Math.max(1, lastConstraints.gridwidth);
                 }
@@ -537,7 +572,7 @@ public class GridBagLayout
                 {
                   if (lastInRow.containsKey(new Integer(y)))
                     {
-                      Component lastComponent = (Component) lastInRow.get(new Integer(y));
+                      Component lastComponent = lastInRow.get(new Integer(y));
                       GridBagConstraints lastConstraints = lookupInternalConstraints(lastComponent);
                       x = Math.max (x,
                                     lastConstraints.gridx + Math.max(1, lastConstraints.gridwidth));
@@ -554,10 +589,12 @@ public class GridBagLayout
               // this column. We want to add this component below it.
               // If this column is empty, add to the 0 position.
               if (!lastInCol.containsKey(new Integer(constraints.gridx))) 
-                y = 0;
+                {
+                  y = current_y;
+                }
               else
                 {
-                  Component lastComponent = (Component)lastInCol.get(new Integer(constraints.gridx));
+                  Component lastComponent = lastInCol.get(new Integer(constraints.gridx));
                   GridBagConstraints lastConstraints = lookupInternalConstraints(lastComponent);
                   y = lastConstraints.gridy + Math.max(1, lastConstraints.gridheight);
                 }
@@ -568,7 +605,7 @@ public class GridBagLayout
                 {
                   if (lastInCol.containsKey(new Integer(x)))
                     {
-                      Component lastComponent = (Component) lastInCol.get(new Integer(x));
+                      Component lastComponent = lastInCol.get(new Integer(x));
                       GridBagConstraints lastConstraints = lookupInternalConstraints(lastComponent);
                       y = Math.max (y,
                                     lastConstraints.gridy + Math.max(1, lastConstraints.gridheight));
@@ -590,7 +627,7 @@ public class GridBagLayout
           // Update our reference points for RELATIVE gridx and gridy.
           if(constraints.gridwidth == GridBagConstraints.REMAINDER)
 	    {
-              current_y = constraints.gridy + Math.max(1, constraints.gridheight);
+          current_y = constraints.gridy + Math.max(1, constraints.gridheight);
 	    }
           else if (constraints.gridwidth != GridBagConstraints.REMAINDER)
 	    {
@@ -598,7 +635,7 @@ public class GridBagLayout
                 {
                   if(lastInRow.containsKey(new Integer(y)))
                     {
-                      Component lastComponent = (Component) lastInRow.get(new Integer(y));
+                      Component lastComponent = lastInRow.get(new Integer(y));
                       GridBagConstraints lastConstraints = lookupInternalConstraints(lastComponent);
                       if (constraints.gridx > lastConstraints.gridx)
                         {
@@ -615,7 +652,7 @@ public class GridBagLayout
                 {
                   if(lastInCol.containsKey(new Integer(x)))
                     {
-                      Component lastComponent = (Component) lastInCol.get(new Integer(x));
+                      Component lastComponent = lastInCol.get(new Integer(x));
                       GridBagConstraints lastConstraints = lookupInternalConstraints(lastComponent);
                       if (constraints.gridy > lastConstraints.gridy)
                         {
@@ -669,7 +706,7 @@ public class GridBagLayout
                     {
                       if (lastInRow.containsKey(new Integer(y)))
                         {
-                          Component lastComponent = (Component) lastInRow.get(new Integer(y));
+                          Component lastComponent = lastInRow.get(new Integer(y));
                           GridBagConstraints lastConstraints = lookupInternalConstraints(lastComponent);
 
                           if (lastConstraints.gridwidth == GridBagConstraints.RELATIVE)
@@ -704,7 +741,7 @@ public class GridBagLayout
                     {
                       if (lastInCol.containsKey(new Integer(x)))
                         {
-                          Component lastComponent = (Component) lastInRow.get(new Integer(x));
+                          Component lastComponent = lastInRow.get(new Integer(x));
                           if (lastComponent != null)
                             {
                               GridBagConstraints lastConstraints = lookupInternalConstraints(lastComponent);
@@ -738,7 +775,7 @@ public class GridBagLayout
       // STEP 3: Determine sizes and weights for columns.
       for (int i = 0; i < sortedByWidth.size(); i++)
         {
-          Component component = (Component) sortedByWidth.get(i);
+          Component component = sortedByWidth.get(i);
 			
           // If component is not visible we dont have to care about it.
           if (!component.isVisible())
@@ -766,7 +803,7 @@ public class GridBagLayout
       // STEP 4: Determine sizes and weights for rows.
       for (int i = 0; i < sortedByHeight.size(); i++)
         {
-          Component component = (Component) sortedByHeight.get(i);
+          Component component = sortedByHeight.get(i);
 			
           // If component is not visible we dont have to care about it.
           if (!component.isVisible())
@@ -782,7 +819,7 @@ public class GridBagLayout
             height += constraints.insets.top + constraints.insets.bottom;
 
           height += constraints.ipady;
-
+          
           distributeSizeAndWeight(height,
                                   constraints.weighty, 
                                   constraints.gridy,
@@ -852,7 +889,8 @@ public class GridBagLayout
      *                    width. Otherwise, sort by height.
      * FIXME: Use a better sorting algorithm.
      */
-    private void sortBySpan (Component component, int span, ArrayList list, boolean sortByWidth)
+    private void sortBySpan (Component component, int span,
+			     ArrayList<Component> list, boolean sortByWidth)
     {
       if (span == GridBagConstraints.REMAINDER
           || span == GridBagConstraints.RELATIVE)
@@ -865,7 +903,7 @@ public class GridBagLayout
           int i = 0;
           if (list.size() > 0)
             {
-              GridBagConstraints gbc = lookupInternalConstraints((Component) list.get(i));
+              GridBagConstraints gbc = lookupInternalConstraints(list.get(i));
               int otherspan = sortByWidth ?
                               gbc.gridwidth :
                               gbc.gridheight;
@@ -876,7 +914,7 @@ public class GridBagLayout
                   i++;
                   if (i < list.size())
                     {
-                      gbc = lookupInternalConstraints((Component) list.get(i));
+                      gbc = lookupInternalConstraints(list.get(i));
                       otherspan = sortByWidth ?
                                   gbc.gridwidth :
                                   gbc.gridheight;
@@ -912,7 +950,7 @@ public class GridBagLayout
           sizes[start] = Math.max(sizes[start], size);
           weights[start] = Math.max(weights[start], weight);
         }
-      else if (span > 1)
+      else
         {
           int numOccupied = span;
           int lastOccupied = -1;
@@ -1059,10 +1097,18 @@ public class GridBagLayout
     }
 
     /**
+     * Move and resize a rectangle according to a set of grid bag
+     * constraints.  The x, y, width and height fields of the
+     * rectangle argument are adjusted to the new values.
+     *
+     * @param constraints position and size constraints
+     * @param r rectangle to be moved and resized
+     *
      * @since 1.4
      */
-    protected void adjustForGravity (GridBagConstraints gbc, Rectangle rect)
+    protected void adjustForGravity (GridBagConstraints constraints,
+                                     Rectangle r)
     {
-      AdjustForGravity (gbc, rect);
+      AdjustForGravity (constraints, r);
     }
 }
